/**
 * Example to show an application that copies the static data section.
 *
 * This allows the program to be reset and restart execution with the
 * initial static data, for example to ensure that the heap is empty.
 */

#include <string.h>
#include <stdio.h>
#include <malloc.h>
#include "mb_interface.h"

/* Labels from the linker script lscript.ld */
extern unsigned char __data_load, __data_start, __data_end;

/* Text buffer in static data section */
char textbuffer[] = "INITIAL TEXT";

int main()
{
    void *pointer;
    volatile int wait;

    /* Initialize data section by copying it from the loaded data section */
    /* See declaration of section .data in the linker script lscript.ld */
    memcpy(&__data_start, &__data_load, &__data_end - &__data_start + 1);

    xil_printf("Starting main\r\n");

    /* Output initial text */
    /* This will always be initial text due to data section initialization above */
    xil_printf("   Text = \"%s\" (should always be \"INITIAL TEXT\")\r\n", textbuffer);

    /* Change text to a new value */
    /* This will be overwritten by data section initialization after starting over */
    strcpy(textbuffer, "CHANGED TEXT");

    /* Allocate memory and output pointer */
    /* This will never fail since data section initialization above will empty heap */
    pointer = malloc(0x400);
    xil_printf("   Memory pointer = 0x%x (should never be 0x0)\r\n", pointer);

    /* Perform a software reset to start over from the beginning */
    /* Wait a while before software reset to ensure text has been output */
    xil_printf("   Restarting main with software reset\r\n\r\n");
    for(wait = 0; wait < 20000; wait++);
    mb_suspend();

    /* The program never gets here */
    return 0;
}
